#include <cybergraphx/cybergraphics.h>
#include	<datatypes/pictureclass.h>
#include	<datatypes/pictureclassext.h>
#include	<devices/rawkeycodes.h>
#include <proto/cybergraphics.h>
#include <proto/datatypes.h>
#include <proto/dos.h>
#include <proto/exec.h>
#include <proto/intuition.h>

/**********************************************************************
	Our version string
**********************************************************************/

CONST TEXT version[] = "\0$VER: LoadPicture 1.0 (1.3.2010)";

/**********************************************************************
	NewDT()

	Load picture using datatypes.
**********************************************************************/

static APTR NewDT(CONST_STRPTR filename, struct BitMapHeader **bmhd)
{
	APTR dtobj;

	dtobj = NewDTObject((STRPTR)filename,
		DTA_GroupID, GID_PICTURE,
		DTA_SourceType, DTST_FILE,
		PDTA_DestMode, PMODE_V43,
		PDTA_FreeSourceBitMap, TRUE,
		TAG_DONE);

	if (dtobj)
	{
		DoMethod(dtobj, DTM_PROCLAYOUT, NULL, 1);
		GetDTAttrs(dtobj, PDTA_BitMapHeader, bmhd, TAG_DONE);
	}

	return dtobj;
}

/**********************************************************************
	EventLoop()

	Wait for user input and return when window is closed or user hits
	ESC or CTRL-C.
**********************************************************************/

static void EventLoop(struct Window *win)
{
	for(;;)
	{
		struct IntuiMessage *msg;
		ULONG signals;

		signals = Wait(1L << win->UserPort->mp_SigBit | SIGBREAKF_CTRL_C);

		if (signals & SIGBREAKF_CTRL_C)
			break;

		while ((msg = (struct IntuiMessage *)GetMsg(win->UserPort)) != NULL)
		{
			ULONG class, code;

			class = msg->Class;
			code  = msg->Code;

			ReplyMsg((struct Message *)msg);

			switch (class)
			{
				case IDCMP_CLOSEWINDOW  : return;
				case IDCMP_RAWKEY       : if (code == RAWKEY_ESCAPE) return; break;
			}
		}
	}
}

/**********************************************************************
	Program's main()
**********************************************************************/

enum
{
	ARG_FILENAME = 0,
	ARG_MAX
};

int main(void)
{
	struct RDArgs *args;
	IPTR params[ARG_MAX];

	params[ARG_FILENAME] = NULL;

	args = ReadArgs("FILENAME/A", (IPTR *)&params, NULL);

	if (args)
	{
		struct BitMapHeader *bmhd;
		APTR dtobj;

		if ((dtobj = NewDT((CONST_STRPTR)params[ARG_FILENAME], &bmhd)))
		{
			struct Window *window;
			ULONG width, height;

			width = bmhd->bmh_Width;
			height = bmhd->bmh_Height;

			window = OpenWindowTags(NULL, WA_InnerWidth, width, WA_InnerHeight, height,
				WA_IDCMP, IDCMP_CLOSEWINDOW | IDCMP_RAWKEY,
				WA_Flags, WFLG_SIMPLE_REFRESH | WFLG_ACTIVATE | WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET,
				WA_Title, (ULONG)"DataTypes",
				TAG_DONE);

			if (window)
			{
				ULONG argbsize, argbmodulo;
				APTR argb;

				argbmodulo = (width * sizeof(ULONG) + 15) & ~15;

				argbsize = argbmodulo * height;

				if ((argb = AllocMem(argbsize, MEMF_ANY)))
				{
					DoMethod(dtobj, PDTM_READPIXELARRAY, argb, PBPAFMT_ARGB, argbmodulo, 0, 0, width, height);

					WritePixelArray(argb, 0, 0, argbmodulo, window->RPort, window->BorderLeft, window->BorderTop, width, height, RECTFMT_ARGB);

					EventLoop(window);

					FreeMem(argb, argbsize);
				}

				CloseWindow(window);
			}

			DisposeDTObject(dtobj);
		}

		FreeArgs(args);
	}

	return 0;
}
